@@ -1,4 +1,4 @@
-// +build nrf52
+//go:build nrf52
 
 package machine
 
@@ -42,28 +42,24 @@ const (
 	P0_31 Pin = 31
 )
 
-var (
-	UART0 = NRF_UART0
-)
-
 // Get peripheral and pin number for this GPIO pin.
 func (p Pin) getPortPin() (*nrf.GPIO_Type, uint32) {
 	return nrf.P0, uint32(p)
 }
 
-func (uart UART) setPins(tx, rx Pin) {
+func (uart *UART) setPins(tx, rx Pin) {
 	nrf.UART0.PSELTXD.Set(uint32(tx))
 	nrf.UART0.PSELRXD.Set(uint32(rx))
 }
 
-func (i2c I2C) setPins(scl, sda Pin) {
+func (i2c *I2C) setPins(scl, sda Pin) {
 	i2c.Bus.PSELSCL.Set(uint32(scl))
 	i2c.Bus.PSELSDA.Set(uint32(sda))
 }
 
 // PWM
 var (
-	pwmChannelPins     = [3]uint32{0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
-	pwms               = [3]*nrf.PWM_Type{nrf.PWM0, nrf.PWM1, nrf.PWM2}
-	pwmChannelSequence [3]uint16
+	PWM0 = &PWM{PWM: nrf.PWM0}
+	PWM1 = &PWM{PWM: nrf.PWM1}
+	PWM2 = &PWM{PWM: nrf.PWM2}
 )
