@@ -1,4 +1,4 @@
-// +build nrf52833
+//go:build nrf52833
 
 package machine
 
@@ -58,10 +58,6 @@ const (
 	P1_15 Pin = 47
 )
 
-var (
-	UART0 = NRF_UART0
-)
-
 // Get peripheral and pin number for this GPIO pin.
 func (p Pin) getPortPin() (*nrf.GPIO_Type, uint32) {
 	if p >= 32 {
@@ -71,7 +67,7 @@ func (p Pin) getPortPin() (*nrf.GPIO_Type, uint32) {
 	}
 }
 
-func (uart UART) setPins(tx, rx Pin) {
+func (uart *UART) setPins(tx, rx Pin) {
 	nrf.UART0.PSEL.TXD.Set(uint32(tx))
 	nrf.UART0.PSEL.RXD.Set(uint32(rx))
 }
