@@ -1,6 +1,7 @@
 package reflect
 
 import (
+	"math"
 	"unsafe"
 )
 
@@ -67,6 +68,14 @@ func (v Value) Interface() interface{} {
 // valueInterfaceUnsafe is used by the runtime to hash map keys. It should not
 // be subject to the isExported check.
 func valueInterfaceUnsafe(v Value) interface{} {
+	if v.typecode.Kind() == Interface {
+		// The value itself is an interface. This can happen when getting the
+		// value of a struct field of interface type, like this:
+		//     type T struct {
+		//         X interface{}
+		//     }
+		return *(*interface{})(v.value)
+	}
 	if v.isIndirect() && v.typecode.Size() <= unsafe.Sizeof(uintptr(0)) {
 		// Value was indirect but must be put back directly in the interface
 		// value.
@@ -83,6 +92,46 @@ func (v Value) Type() Type {
 	return v.typecode
 }
 
+// IsZero reports whether v is the zero value for its type.
+// It panics if the argument is invalid.
+func (v Value) IsZero() bool {
+	switch v.Kind() {
+	case Bool:
+		return !v.Bool()
+	case Int, Int8, Int16, Int32, Int64:
+		return v.Int() == 0
+	case Uint, Uint8, Uint16, Uint32, Uint64, Uintptr:
+		return v.Uint() == 0
+	case Float32, Float64:
+		return math.Float64bits(v.Float()) == 0
+	case Complex64, Complex128:
+		c := v.Complex()
+		return math.Float64bits(real(c)) == 0 && math.Float64bits(imag(c)) == 0
+	case Array:
+		for i := 0; i < v.Len(); i++ {
+			if !v.Index(i).IsZero() {
+				return false
+			}
+		}
+		return true
+	case Chan, Func, Interface, Map, Pointer, Slice, UnsafePointer:
+		return v.IsNil()
+	case String:
+		return v.Len() == 0
+	case Struct:
+		for i := 0; i < v.NumField(); i++ {
+			if !v.Field(i).IsZero() {
+				return false
+			}
+		}
+		return true
+	default:
+		// This should never happens, but will act as a safeguard for
+		// later, as a default value doesn't makes sense here.
+		panic(&ValueError{"reflect.Value.IsZero", v.Kind()})
+	}
+}
+
 // Internal function only, do not use.
 //
 // RawType returns the raw, underlying type code. It is used in the runtime
@@ -99,11 +148,8 @@ func (v Value) Kind() Kind {
 // is not a channel, map, pointer, function, slice, or interface.
 func (v Value) IsNil() bool {
 	switch v.Kind() {
-	case Chan, Map, Ptr:
-		if v.isIndirect() {
-			return *(*uintptr)(v.value) == 0
-		}
-		return v.value == nil
+	case Chan, Map, Ptr, UnsafePointer:
+		return v.pointer() == nil
 	case Func:
 		if v.value == nil {
 			return true
@@ -123,36 +169,47 @@ func (v Value) IsNil() bool {
 		_, val := decomposeInterface(*(*interface{})(v.value))
 		return val == nil
 	default:
-		panic(&ValueError{"IsNil"})
+		panic(&ValueError{Method: "IsNil"})
 	}
 }
 
 // Pointer returns the underlying pointer of the given value for the following
 // types: chan, map, pointer, unsafe.Pointer, slice, func.
 func (v Value) Pointer() uintptr {
+	return uintptr(v.UnsafePointer())
+}
+
+// UnsafePointer returns the underlying pointer of the given value for the
+// following types: chan, map, pointer, unsafe.Pointer, slice, func.
+func (v Value) UnsafePointer() unsafe.Pointer {
 	switch v.Kind() {
 	case Chan, Map, Ptr, UnsafePointer:
-		if v.isIndirect() {
-			return *(*uintptr)(v.value)
-		}
-		return uintptr(v.value)
+		return v.pointer()
 	case Slice:
 		slice := (*sliceHeader)(v.value)
-		return uintptr(slice.data)
+		return slice.data
 	case Func:
-		panic("unimplemented: (reflect.Value).Pointer()")
+		panic("unimplemented: (reflect.Value).UnsafePointer()")
 	default: // not implemented: Func
-		panic(&ValueError{"Pointer"})
+		panic(&ValueError{Method: "UnsafePointer"})
 	}
 }
 
+// pointer returns the underlying pointer represented by v.
+// v.Kind() must be Ptr, Map, Chan, or UnsafePointer
+func (v Value) pointer() unsafe.Pointer {
+	if v.isIndirect() {
+		return *(*unsafe.Pointer)(v.value)
+	}
+	return v.value
+}
+
 func (v Value) IsValid() bool {
 	return v.typecode != 0
 }
 
 func (v Value) CanInterface() bool {
-	// No Value types of private data can be constructed at the moment.
-	return true
+	return v.isExported()
 }
 
 func (v Value) CanAddr() bool {
@@ -176,7 +233,7 @@ func (v Value) Bool() bool {
 			return uintptr(v.value) != 0
 		}
 	default:
-		panic(&ValueError{"Bool"})
+		panic(&ValueError{Method: "Bool"})
 	}
 }
 
@@ -213,7 +270,7 @@ func (v Value) Int() int64 {
 			return int64(int64(uintptr(v.value)))
 		}
 	default:
-		panic(&ValueError{"Int"})
+		panic(&ValueError{Method: "Int"})
 	}
 }
 
@@ -256,7 +313,7 @@ func (v Value) Uint() uint64 {
 			return uint64(uintptr(v.value))
 		}
 	default:
-		panic(&ValueError{"Uint"})
+		panic(&ValueError{Method: "Uint"})
 	}
 }
 
@@ -282,7 +339,7 @@ func (v Value) Float() float64 {
 			return *(*float64)(unsafe.Pointer(&v.value))
 		}
 	default:
-		panic(&ValueError{"Float"})
+		panic(&ValueError{Method: "Float"})
 	}
 }
 
@@ -304,7 +361,7 @@ func (v Value) Complex() complex128 {
 		// architectures with 128-bit pointers, however.
 		return *(*complex128)(v.value)
 	default:
-		panic(&ValueError{"Complex"})
+		panic(&ValueError{Method: "Complex"})
 	}
 }
 
@@ -328,6 +385,10 @@ func (v Value) Slice(i, j int) Value {
 	panic("unimplemented: (reflect.Value).Slice()")
 }
 
+func (v Value) Slice3(i, j, k int) Value {
+	panic("unimplemented: (reflect.Value).Slice3()")
+}
+
 //go:linkname maplen runtime.hashmapLenUnsafePointer
 func maplen(p unsafe.Pointer) int
 
@@ -341,15 +402,15 @@ func (v Value) Len() int {
 	case Array:
 		return v.typecode.Len()
 	case Chan:
-		return chanlen(v.value)
+		return chanlen(v.pointer())
 	case Map:
-		return maplen(v.value)
+		return maplen(v.pointer())
 	case Slice:
 		return int((*sliceHeader)(v.value).len)
 	case String:
 		return int((*stringHeader)(v.value).len)
 	default:
-		panic(&ValueError{"Len"})
+		panic(&ValueError{Method: "Len"})
 	}
 }
 
@@ -363,11 +424,11 @@ func (v Value) Cap() int {
 	case Array:
 		return v.typecode.Len()
 	case Chan:
-		return chancap(v.value)
+		return chancap(v.pointer())
 	case Slice:
 		return int((*sliceHeader)(v.value).cap)
 	default:
-		panic(&ValueError{"Cap"})
+		panic(&ValueError{Method: "Cap"})
 	}
 }
 
@@ -380,10 +441,7 @@ func (v Value) NumField() int {
 func (v Value) Elem() Value {
 	switch v.Kind() {
 	case Ptr:
-		ptr := v.value
-		if v.isIndirect() {
-			ptr = *(*unsafe.Pointer)(ptr)
-		}
+		ptr := v.pointer()
 		if ptr == nil {
 			return Value{}
 		}
@@ -392,8 +450,15 @@ func (v Value) Elem() Value {
 			value:    ptr,
 			flags:    v.flags | valueFlagIndirect,
 		}
-	default: // not implemented: Interface
-		panic(&ValueError{"Elem"})
+	case Interface:
+		typecode, value := decomposeInterface(*(*interface{})(v.value))
+		return Value{
+			typecode: typecode,
+			value:    value,
+			flags:    v.flags &^ valueFlagIndirect,
+		}
+	default:
+		panic(&ValueError{Method: "Elem"})
 	}
 }
 
@@ -442,7 +507,7 @@ func (v Value) Field(i int) Value {
 		ptr := unsafe.Pointer(uintptr(v.value) + structField.Offset)
 		value := unsafe.Pointer(loadValue(ptr, fieldSize))
 		return Value{
-			flags:    0,
+			flags:    flags &^ valueFlagIndirect,
 			typecode: fieldType,
 			value:    value,
 		}
@@ -477,6 +542,9 @@ func (v Value) Index(i int) Value {
 		// Extract a character from a string.
 		// A string is never stored directly in the interface, but always as a
 		// pointer to the string value.
+		// Keeping valueFlagExported if set, but don't set valueFlagIndirect
+		// otherwise CanSet will return true for string elements (which is bad,
+		// strings are read-only).
 		s := *(*stringHeader)(v.value)
 		if uint(i) >= uint(s.len) {
 			panic("reflect: string index out of range")
@@ -484,6 +552,7 @@ func (v Value) Index(i int) Value {
 		return Value{
 			typecode: Uint8.basicType(),
 			value:    unsafe.Pointer(uintptr(*(*uint8)(unsafe.Pointer(uintptr(s.data) + uintptr(i))))),
+			flags:    v.flags & valueFlagExported,
 		}
 	case Array:
 		// Extract an element from the array.
@@ -510,14 +579,20 @@ func (v Value) Index(i int) Value {
 			}
 		}
 
-		if size > unsafe.Sizeof(uintptr(0)) {
-			// The element fits in a pointer, but the array does not.
+		if size > unsafe.Sizeof(uintptr(0)) || v.isIndirect() {
+			// The element fits in a pointer, but the array is not stored in the pointer directly.
 			// Load the value from the pointer.
-			addr := uintptr(v.value) + elemSize*uintptr(i) // pointer to new value
+			addr := unsafe.Pointer(uintptr(v.value) + elemSize*uintptr(i)) // pointer to new value
+			value := addr
+			if !v.isIndirect() {
+				// Use a pointer to the value (don't load the value) if the
+				// 'indirect' flag is set.
+				value = unsafe.Pointer(loadValue(addr, elemSize))
+			}
 			return Value{
 				typecode: v.typecode.elem(),
 				flags:    v.flags,
-				value:    unsafe.Pointer(loadValue(unsafe.Pointer(addr), elemSize)),
+				value:    value,
 			}
 		}
 
@@ -531,7 +606,7 @@ func (v Value) Index(i int) Value {
 			value:    unsafe.Pointer(value),
 		}
 	default:
-		panic(&ValueError{"Index"})
+		panic(&ValueError{Method: "Index"})
 	}
 }
 
@@ -610,7 +685,7 @@ func (v Value) SetBool(x bool) {
 	case Bool:
 		*(*bool)(v.value) = x
 	default:
-		panic(&ValueError{"SetBool"})
+		panic(&ValueError{Method: "SetBool"})
 	}
 }
 
@@ -628,7 +703,7 @@ func (v Value) SetInt(x int64) {
 	case Int64:
 		*(*int64)(v.value) = x
 	default:
-		panic(&ValueError{"SetInt"})
+		panic(&ValueError{Method: "SetInt"})
 	}
 }
 
@@ -648,7 +723,7 @@ func (v Value) SetUint(x uint64) {
 	case Uintptr:
 		*(*uintptr)(v.value) = uintptr(x)
 	default:
-		panic(&ValueError{"SetUint"})
+		panic(&ValueError{Method: "SetUint"})
 	}
 }
 
@@ -660,7 +735,7 @@ func (v Value) SetFloat(x float64) {
 	case Float64:
 		*(*float64)(v.value) = x
 	default:
-		panic(&ValueError{"SetFloat"})
+		panic(&ValueError{Method: "SetFloat"})
 	}
 }
 
@@ -672,7 +747,7 @@ func (v Value) SetComplex(x complex128) {
 	case Complex128:
 		*(*complex128)(v.value) = x
 	default:
-		panic(&ValueError{"SetComplex"})
+		panic(&ValueError{Method: "SetComplex"})
 	}
 }
 
@@ -682,7 +757,7 @@ func (v Value) SetString(x string) {
 	case String:
 		*(*string)(v.value) = x
 	default:
-		panic(&ValueError{"SetString"})
+		panic(&ValueError{Method: "SetString"})
 	}
 }
 
@@ -729,7 +804,7 @@ func Zero(typ Type) Value {
 func New(typ Type) Value {
 	return Value{
 		typecode: PtrTo(typ).(rawType),
-		value:    alloc(typ.Size()),
+		value:    alloc(typ.Size(), nil),
 		flags:    valueFlagExported,
 	}
 }
@@ -767,18 +842,21 @@ type stringHeader struct {
 
 type ValueError struct {
 	Method string
+	Kind   Kind
 }
 
 func (e *ValueError) Error() string {
-	return "reflect: call of reflect.Value." + e.Method + " on invalid type"
+	if e.Kind == 0 {
+		return "reflect: call of " + e.Method + " on zero Value"
+	}
+	return "reflect: call of " + e.Method + " on " + e.Kind.String() + " Value"
 }
 
-// Calls to this function are converted to LLVM intrinsic calls such as
-// llvm.memcpy.p0i8.p0i8.i32().
+//go:linkname memcpy runtime.memcpy
 func memcpy(dst, src unsafe.Pointer, size uintptr)
 
 //go:linkname alloc runtime.alloc
-func alloc(size uintptr) unsafe.Pointer
+func alloc(size uintptr, layout unsafe.Pointer) unsafe.Pointer
 
 //go:linkname sliceAppend runtime.sliceAppend
 func sliceAppend(srcBuf, elemsBuf unsafe.Pointer, srcLen, srcCap, elemsLen uintptr, elemSize uintptr) (unsafe.Pointer, uintptr, uintptr)
@@ -832,6 +910,11 @@ func (v Value) FieldByIndex(index []int) Value {
 	panic("unimplemented: (reflect.Value).FieldByIndex()")
 }
 
+// FieldByIndexErr returns the nested field corresponding to index.
+func (v Value) FieldByIndexErr(index []int) (Value, error) {
+	return Value{}, &ValueError{Method: "FieldByIndexErr"}
+}
+
 func (v Value) FieldByName(name string) Value {
 	panic("unimplemented: (reflect.Value).FieldByName()")
 }
@@ -844,3 +927,15 @@ func MakeMap(typ Type) Value {
 func (v Value) Call(in []Value) []Value {
 	panic("unimplemented: (reflect.Value).Call()")
 }
+
+func (v Value) MethodByName(name string) Value {
+	panic("unimplemented: (reflect.Value).MethodByName()")
+}
+
+func (v Value) Recv() (x Value, ok bool) {
+	panic("unimplemented: (reflect.Value).Recv()")
+}
+
+func NewAt(typ Type, p unsafe.Pointer) Value {
+	panic("unimplemented: reflect.New()")
+}
